#define SOLARUS_MAJOR_VERSION 1
#define SOLARUS_MINOR_VERSION 6
/* #undef SOLARUS_PATCH_VERSION */

#ifndef SOLARUS_MINOR_VERSION
#define SOLARUS_MINOR_VERSION 0
#endif

#ifndef SOLARUS_PATCH_VERSION
#define SOLARUS_PATCH_VERSION 0
#endif

#define SOLARUS_HAVE_OPENGL 1
#define SOLARUS_HAVE_MKSTEMP 1
#define SOLARUS_HAVE_UNISTD_H 1

