/********************************************************************************
** Form generated from reading UI file 'main_window.ui'
**
** Created by: Qt User Interface Compiler version 5.15.18
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_MAIN_WINDOW_H
#define UI_MAIN_WINDOW_H

#include <QtCore/QVariant>
#include <QtGui/QIcon>
#include <QtWidgets/QAction>
#include <QtWidgets/QApplication>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QGroupBox>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QHeaderView>
#include <QtWidgets/QLabel>
#include <QtWidgets/QMainWindow>
#include <QtWidgets/QMenu>
#include <QtWidgets/QMenuBar>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QScrollArea>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QSplitter>
#include <QtWidgets/QToolButton>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>
#include "solarus/gui/console.h"
#include "solarus/gui/quests_view.h"

namespace SolarusGui {

class Ui_MainWindow
{
public:
    QAction *action_add_quest;
    QAction *action_exit;
    QAction *action_play_quest;
    QAction *action_remove_quest;
    QAction *action_stop_quest;
    QAction *action_sound_volume;
    QAction *action_music_volume;
    QAction *action_fullscreen;
    QAction *action_zoom_x1;
    QAction *action_zoom_x2;
    QAction *action_zoom_x3;
    QAction *action_zoom_x4;
    QAction *action_mute;
    QAction *action_about;
    QWidget *centralwidget;
    QVBoxLayout *verticalLayout_3;
    QSplitter *console_splitter;
    QWidget *top_widget;
    QGridLayout *top_widget_layout;
    QHBoxLayout *quest_buttons_layout;
    QSpacerItem *horizontalSpacer;
    QToolButton *add_button;
    QToolButton *remove_button;
    SolarusGui::QuestsView *quests_view;
    QScrollArea *quest_info_scrollarea;
    QWidget *quest_info_panel;
    QVBoxLayout *verticalLayout_2;
    QWidget *quest_box_container;
    QHBoxLayout *horizontalLayout;
    QSpacerItem *horizontalSpacer_2;
    QLabel *quest_box_label;
    QSpacerItem *horizontalSpacer_3;
    QHBoxLayout *play_button_layout;
    QSpacerItem *play_button_left_spacer;
    QPushButton *play_button;
    QSpacerItem *playbutton_right_spacer;
    QLabel *quest_title_value;
    QLabel *quest_long_description_value;
    QGridLayout *quest_infos_layout;
    QLabel *quest_author_label;
    QLabel *quest_author_value;
    QLabel *quest_version_label;
    QLabel *quest_version_value;
    QSpacerItem *quest_info_vertical_spacer;
    QWidget *bottom_widget;
    QVBoxLayout *bottom_widget_layout;
    QGroupBox *groupBox;
    QVBoxLayout *verticalLayout_4;
    SolarusGui::Console *console;
    QMenuBar *menu_bar;
    QMenu *menu_file;
    QMenu *menu_quest;
    QMenu *menu_video;
    QMenu *menu_zoom;
    QMenu *menu_audio;
    QMenu *menu_help;

    void setupUi(QMainWindow *SolarusGui__MainWindow)
    {
        if (SolarusGui__MainWindow->objectName().isEmpty())
            SolarusGui__MainWindow->setObjectName(QString::fromUtf8("SolarusGui__MainWindow"));
        SolarusGui__MainWindow->resize(960, 640);
        SolarusGui__MainWindow->setMinimumSize(QSize(600, 0));
        SolarusGui__MainWindow->setStyleSheet(QString::fromUtf8("QLabel#quest_title_value {\n"
"	font-size: 13pt;\n"
"	font-weight: bold;\n"
"}\n"
"\n"
"QLabel#quest_author_label,\n"
"QLabel#quest_version_label {\n"
"	font-weight: bold;\n"
"}\n"
"\n"
"QPushButton#play_button {\n"
"	font-weight: bold;\n"
"}"));
        action_add_quest = new QAction(SolarusGui__MainWindow);
        action_add_quest->setObjectName(QString::fromUtf8("action_add_quest"));
        QIcon icon;
        icon.addFile(QString::fromUtf8(":/images/icon_add.png"), QSize(), QIcon::Normal, QIcon::Off);
        action_add_quest->setIcon(icon);
        action_exit = new QAction(SolarusGui__MainWindow);
        action_exit->setObjectName(QString::fromUtf8("action_exit"));
        action_play_quest = new QAction(SolarusGui__MainWindow);
        action_play_quest->setObjectName(QString::fromUtf8("action_play_quest"));
        QIcon icon1;
        icon1.addFile(QString::fromUtf8(":/images/icon_start.png"), QSize(), QIcon::Normal, QIcon::Off);
        action_play_quest->setIcon(icon1);
        action_remove_quest = new QAction(SolarusGui__MainWindow);
        action_remove_quest->setObjectName(QString::fromUtf8("action_remove_quest"));
        QIcon icon2;
        icon2.addFile(QString::fromUtf8(":/images/icon_remove.png"), QSize(), QIcon::Normal, QIcon::Off);
        action_remove_quest->setIcon(icon2);
        action_stop_quest = new QAction(SolarusGui__MainWindow);
        action_stop_quest->setObjectName(QString::fromUtf8("action_stop_quest"));
        QIcon icon3;
        icon3.addFile(QString::fromUtf8(":/images/icon_stop.png"), QSize(), QIcon::Normal, QIcon::Off);
        action_stop_quest->setIcon(icon3);
        action_sound_volume = new QAction(SolarusGui__MainWindow);
        action_sound_volume->setObjectName(QString::fromUtf8("action_sound_volume"));
        action_music_volume = new QAction(SolarusGui__MainWindow);
        action_music_volume->setObjectName(QString::fromUtf8("action_music_volume"));
        action_fullscreen = new QAction(SolarusGui__MainWindow);
        action_fullscreen->setObjectName(QString::fromUtf8("action_fullscreen"));
        action_fullscreen->setCheckable(true);
        action_zoom_x1 = new QAction(SolarusGui__MainWindow);
        action_zoom_x1->setObjectName(QString::fromUtf8("action_zoom_x1"));
        action_zoom_x2 = new QAction(SolarusGui__MainWindow);
        action_zoom_x2->setObjectName(QString::fromUtf8("action_zoom_x2"));
        action_zoom_x3 = new QAction(SolarusGui__MainWindow);
        action_zoom_x3->setObjectName(QString::fromUtf8("action_zoom_x3"));
        action_zoom_x4 = new QAction(SolarusGui__MainWindow);
        action_zoom_x4->setObjectName(QString::fromUtf8("action_zoom_x4"));
        action_mute = new QAction(SolarusGui__MainWindow);
        action_mute->setObjectName(QString::fromUtf8("action_mute"));
        action_mute->setCheckable(true);
        action_about = new QAction(SolarusGui__MainWindow);
        action_about->setObjectName(QString::fromUtf8("action_about"));
        centralwidget = new QWidget(SolarusGui__MainWindow);
        centralwidget->setObjectName(QString::fromUtf8("centralwidget"));
        verticalLayout_3 = new QVBoxLayout(centralwidget);
        verticalLayout_3->setObjectName(QString::fromUtf8("verticalLayout_3"));
        console_splitter = new QSplitter(centralwidget);
        console_splitter->setObjectName(QString::fromUtf8("console_splitter"));
        console_splitter->setOrientation(Qt::Vertical);
        console_splitter->setHandleWidth(9);
        top_widget = new QWidget(console_splitter);
        top_widget->setObjectName(QString::fromUtf8("top_widget"));
        top_widget_layout = new QGridLayout(top_widget);
        top_widget_layout->setSpacing(6);
        top_widget_layout->setObjectName(QString::fromUtf8("top_widget_layout"));
        top_widget_layout->setContentsMargins(0, 0, 0, 0);
        quest_buttons_layout = new QHBoxLayout();
        quest_buttons_layout->setSpacing(3);
        quest_buttons_layout->setObjectName(QString::fromUtf8("quest_buttons_layout"));
        quest_buttons_layout->setContentsMargins(0, 0, 0, 0);
        horizontalSpacer = new QSpacerItem(0, 0, QSizePolicy::Expanding, QSizePolicy::Minimum);

        quest_buttons_layout->addItem(horizontalSpacer);

        add_button = new QToolButton(top_widget);
        add_button->setObjectName(QString::fromUtf8("add_button"));
        QSizePolicy sizePolicy(QSizePolicy::Fixed, QSizePolicy::Fixed);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(add_button->sizePolicy().hasHeightForWidth());
        add_button->setSizePolicy(sizePolicy);
        add_button->setMinimumSize(QSize(24, 24));
        add_button->setIcon(icon);
        add_button->setIconSize(QSize(16, 16));
        add_button->setToolButtonStyle(Qt::ToolButtonTextBesideIcon);

        quest_buttons_layout->addWidget(add_button);

        remove_button = new QToolButton(top_widget);
        remove_button->setObjectName(QString::fromUtf8("remove_button"));
        sizePolicy.setHeightForWidth(remove_button->sizePolicy().hasHeightForWidth());
        remove_button->setSizePolicy(sizePolicy);
        remove_button->setMinimumSize(QSize(24, 24));
        remove_button->setIcon(icon2);
        remove_button->setIconSize(QSize(16, 16));
        remove_button->setToolButtonStyle(Qt::ToolButtonTextBesideIcon);

        quest_buttons_layout->addWidget(remove_button);

        quest_buttons_layout->setStretch(1, 1);
        quest_buttons_layout->setStretch(2, 1);

        top_widget_layout->addLayout(quest_buttons_layout, 2, 0, 1, 1);

        quests_view = new SolarusGui::QuestsView(top_widget);
        quests_view->setObjectName(QString::fromUtf8("quests_view"));
        quests_view->setEditTriggers(QAbstractItemView::NoEditTriggers);
        quests_view->setSelectionMode(QAbstractItemView::SingleSelection);
        quests_view->setSelectionBehavior(QAbstractItemView::SelectRows);
        quests_view->horizontalHeader()->setMinimumSectionSize(80);
        quests_view->horizontalHeader()->setDefaultSectionSize(80);
        quests_view->horizontalHeader()->setHighlightSections(false);
        quests_view->horizontalHeader()->setProperty("showSortIndicator", QVariant(true));
        quests_view->verticalHeader()->setVisible(false);
        quests_view->verticalHeader()->setMinimumSectionSize(50);
        quests_view->verticalHeader()->setDefaultSectionSize(50);

        top_widget_layout->addWidget(quests_view, 0, 0, 1, 1);

        quest_info_scrollarea = new QScrollArea(top_widget);
        quest_info_scrollarea->setObjectName(QString::fromUtf8("quest_info_scrollarea"));
        QSizePolicy sizePolicy1(QSizePolicy::Expanding, QSizePolicy::Expanding);
        sizePolicy1.setHorizontalStretch(0);
        sizePolicy1.setVerticalStretch(0);
        sizePolicy1.setHeightForWidth(quest_info_scrollarea->sizePolicy().hasHeightForWidth());
        quest_info_scrollarea->setSizePolicy(sizePolicy1);
        quest_info_scrollarea->setMinimumSize(QSize(260, 340));
        quest_info_scrollarea->setMaximumSize(QSize(260, 16777215));
        quest_info_scrollarea->setFrameShape(QFrame::NoFrame);
        quest_info_scrollarea->setFrameShadow(QFrame::Plain);
        quest_info_scrollarea->setVerticalScrollBarPolicy(Qt::ScrollBarAsNeeded);
        quest_info_scrollarea->setHorizontalScrollBarPolicy(Qt::ScrollBarAlwaysOff);
        quest_info_scrollarea->setWidgetResizable(true);
        quest_info_scrollarea->setAlignment(Qt::AlignCenter);
        quest_info_panel = new QWidget();
        quest_info_panel->setObjectName(QString::fromUtf8("quest_info_panel"));
        quest_info_panel->setGeometry(QRect(0, 0, 260, 464));
        QSizePolicy sizePolicy2(QSizePolicy::Preferred, QSizePolicy::Preferred);
        sizePolicy2.setHorizontalStretch(0);
        sizePolicy2.setVerticalStretch(0);
        sizePolicy2.setHeightForWidth(quest_info_panel->sizePolicy().hasHeightForWidth());
        quest_info_panel->setSizePolicy(sizePolicy2);
        verticalLayout_2 = new QVBoxLayout(quest_info_panel);
        verticalLayout_2->setSpacing(9);
        verticalLayout_2->setObjectName(QString::fromUtf8("verticalLayout_2"));
        verticalLayout_2->setContentsMargins(9, 3, 9, 3);
        quest_box_container = new QWidget(quest_info_panel);
        quest_box_container->setObjectName(QString::fromUtf8("quest_box_container"));
        QSizePolicy sizePolicy3(QSizePolicy::Preferred, QSizePolicy::Fixed);
        sizePolicy3.setHorizontalStretch(0);
        sizePolicy3.setVerticalStretch(0);
        sizePolicy3.setHeightForWidth(quest_box_container->sizePolicy().hasHeightForWidth());
        quest_box_container->setSizePolicy(sizePolicy3);
        quest_box_container->setMinimumSize(QSize(0, 140));
        quest_box_container->setMaximumSize(QSize(16777215, 140));
        quest_box_container->setFocusPolicy(Qt::NoFocus);
        horizontalLayout = new QHBoxLayout(quest_box_container);
        horizontalLayout->setSpacing(0);
        horizontalLayout->setObjectName(QString::fromUtf8("horizontalLayout"));
        horizontalLayout->setSizeConstraint(QLayout::SetDefaultConstraint);
        horizontalLayout->setContentsMargins(0, 0, 0, 0);
        horizontalSpacer_2 = new QSpacerItem(0, 0, QSizePolicy::Expanding, QSizePolicy::Minimum);

        horizontalLayout->addItem(horizontalSpacer_2);

        quest_box_label = new QLabel(quest_box_container);
        quest_box_label->setObjectName(QString::fromUtf8("quest_box_label"));
        quest_box_label->setMinimumSize(QSize(200, 140));
        quest_box_label->setFocusPolicy(Qt::NoFocus);

        horizontalLayout->addWidget(quest_box_label);

        horizontalSpacer_3 = new QSpacerItem(0, 0, QSizePolicy::Expanding, QSizePolicy::Minimum);

        horizontalLayout->addItem(horizontalSpacer_3);


        verticalLayout_2->addWidget(quest_box_container);

        play_button_layout = new QHBoxLayout();
        play_button_layout->setSpacing(0);
        play_button_layout->setObjectName(QString::fromUtf8("play_button_layout"));
        play_button_layout->setSizeConstraint(QLayout::SetDefaultConstraint);
        play_button_layout->setContentsMargins(0, 0, 0, 0);
        play_button_left_spacer = new QSpacerItem(0, 0, QSizePolicy::Expanding, QSizePolicy::Minimum);

        play_button_layout->addItem(play_button_left_spacer);

        play_button = new QPushButton(quest_info_panel);
        play_button->setObjectName(QString::fromUtf8("play_button"));
        sizePolicy.setHeightForWidth(play_button->sizePolicy().hasHeightForWidth());
        play_button->setSizePolicy(sizePolicy);
        play_button->setIcon(icon1);
        play_button->setIconSize(QSize(16, 16));
        play_button->setAutoDefault(false);

        play_button_layout->addWidget(play_button);

        playbutton_right_spacer = new QSpacerItem(0, 0, QSizePolicy::Expanding, QSizePolicy::Minimum);

        play_button_layout->addItem(playbutton_right_spacer);


        verticalLayout_2->addLayout(play_button_layout);

        quest_title_value = new QLabel(quest_info_panel);
        quest_title_value->setObjectName(QString::fromUtf8("quest_title_value"));
        sizePolicy3.setHeightForWidth(quest_title_value->sizePolicy().hasHeightForWidth());
        quest_title_value->setSizePolicy(sizePolicy3);
        quest_title_value->setAlignment(Qt::AlignHCenter|Qt::AlignTop);
        quest_title_value->setWordWrap(true);

        verticalLayout_2->addWidget(quest_title_value);

        quest_long_description_value = new QLabel(quest_info_panel);
        quest_long_description_value->setObjectName(QString::fromUtf8("quest_long_description_value"));
        sizePolicy2.setHeightForWidth(quest_long_description_value->sizePolicy().hasHeightForWidth());
        quest_long_description_value->setSizePolicy(sizePolicy2);
        quest_long_description_value->setAlignment(Qt::AlignHCenter|Qt::AlignTop);
        quest_long_description_value->setWordWrap(true);

        verticalLayout_2->addWidget(quest_long_description_value);

        quest_infos_layout = new QGridLayout();
        quest_infos_layout->setObjectName(QString::fromUtf8("quest_infos_layout"));
        quest_infos_layout->setSizeConstraint(QLayout::SetDefaultConstraint);
        quest_infos_layout->setHorizontalSpacing(6);
        quest_infos_layout->setVerticalSpacing(1);
        quest_infos_layout->setContentsMargins(0, 0, 0, 0);
        quest_author_label = new QLabel(quest_info_panel);
        quest_author_label->setObjectName(QString::fromUtf8("quest_author_label"));
        quest_author_label->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);

        quest_infos_layout->addWidget(quest_author_label, 0, 0, 1, 1);

        quest_author_value = new QLabel(quest_info_panel);
        quest_author_value->setObjectName(QString::fromUtf8("quest_author_value"));
        sizePolicy3.setHeightForWidth(quest_author_value->sizePolicy().hasHeightForWidth());
        quest_author_value->setSizePolicy(sizePolicy3);
        quest_author_value->setWordWrap(true);

        quest_infos_layout->addWidget(quest_author_value, 0, 1, 1, 1);

        quest_version_label = new QLabel(quest_info_panel);
        quest_version_label->setObjectName(QString::fromUtf8("quest_version_label"));
        quest_version_label->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);

        quest_infos_layout->addWidget(quest_version_label, 1, 0, 1, 1);

        quest_version_value = new QLabel(quest_info_panel);
        quest_version_value->setObjectName(QString::fromUtf8("quest_version_value"));
        sizePolicy3.setHeightForWidth(quest_version_value->sizePolicy().hasHeightForWidth());
        quest_version_value->setSizePolicy(sizePolicy3);
        quest_version_value->setWordWrap(true);

        quest_infos_layout->addWidget(quest_version_value, 1, 1, 1, 1);

        quest_infos_layout->setColumnStretch(1, 1);

        verticalLayout_2->addLayout(quest_infos_layout);

        quest_info_vertical_spacer = new QSpacerItem(0, 0, QSizePolicy::Minimum, QSizePolicy::Expanding);

        verticalLayout_2->addItem(quest_info_vertical_spacer);

        quest_info_scrollarea->setWidget(quest_info_panel);

        top_widget_layout->addWidget(quest_info_scrollarea, 0, 1, 1, 1);

        console_splitter->addWidget(top_widget);
        bottom_widget = new QWidget(console_splitter);
        bottom_widget->setObjectName(QString::fromUtf8("bottom_widget"));
        bottom_widget_layout = new QVBoxLayout(bottom_widget);
        bottom_widget_layout->setSpacing(6);
        bottom_widget_layout->setObjectName(QString::fromUtf8("bottom_widget_layout"));
        bottom_widget_layout->setContentsMargins(0, 0, 0, 0);
        groupBox = new QGroupBox(bottom_widget);
        groupBox->setObjectName(QString::fromUtf8("groupBox"));
        verticalLayout_4 = new QVBoxLayout(groupBox);
        verticalLayout_4->setObjectName(QString::fromUtf8("verticalLayout_4"));
        console = new SolarusGui::Console(groupBox);
        console->setObjectName(QString::fromUtf8("console"));
        console->setMinimumSize(QSize(0, 30));

        verticalLayout_4->addWidget(console);


        bottom_widget_layout->addWidget(groupBox);

        console_splitter->addWidget(bottom_widget);

        verticalLayout_3->addWidget(console_splitter);

        SolarusGui__MainWindow->setCentralWidget(centralwidget);
        menu_bar = new QMenuBar(SolarusGui__MainWindow);
        menu_bar->setObjectName(QString::fromUtf8("menu_bar"));
        menu_bar->setGeometry(QRect(0, 0, 960, 22));
        menu_file = new QMenu(menu_bar);
        menu_file->setObjectName(QString::fromUtf8("menu_file"));
        menu_quest = new QMenu(menu_bar);
        menu_quest->setObjectName(QString::fromUtf8("menu_quest"));
        menu_video = new QMenu(menu_bar);
        menu_video->setObjectName(QString::fromUtf8("menu_video"));
        menu_zoom = new QMenu(menu_video);
        menu_zoom->setObjectName(QString::fromUtf8("menu_zoom"));
        menu_audio = new QMenu(menu_bar);
        menu_audio->setObjectName(QString::fromUtf8("menu_audio"));
        menu_help = new QMenu(menu_bar);
        menu_help->setObjectName(QString::fromUtf8("menu_help"));
        SolarusGui__MainWindow->setMenuBar(menu_bar);
        QWidget::setTabOrder(add_button, remove_button);
        QWidget::setTabOrder(remove_button, play_button);

        menu_bar->addAction(menu_file->menuAction());
        menu_bar->addAction(menu_quest->menuAction());
        menu_bar->addAction(menu_video->menuAction());
        menu_bar->addAction(menu_audio->menuAction());
        menu_bar->addAction(menu_help->menuAction());
        menu_file->addAction(action_add_quest);
        menu_file->addAction(action_remove_quest);
        menu_file->addSeparator();
        menu_file->addAction(action_exit);
        menu_quest->addAction(action_play_quest);
        menu_quest->addAction(action_stop_quest);
        menu_quest->addSeparator();
        menu_video->addAction(action_fullscreen);
        menu_video->addAction(menu_zoom->menuAction());
        menu_zoom->addAction(action_zoom_x1);
        menu_zoom->addAction(action_zoom_x2);
        menu_zoom->addAction(action_zoom_x3);
        menu_zoom->addAction(action_zoom_x4);
        menu_audio->addAction(action_mute);
        menu_audio->addAction(action_music_volume);
        menu_audio->addAction(action_sound_volume);
        menu_help->addAction(action_about);

        retranslateUi(SolarusGui__MainWindow);

        play_button->setDefault(false);


        QMetaObject::connectSlotsByName(SolarusGui__MainWindow);
    } // setupUi

    void retranslateUi(QMainWindow *SolarusGui__MainWindow)
    {
        SolarusGui__MainWindow->setWindowTitle(QCoreApplication::translate("SolarusGui::MainWindow", "Solarus", nullptr));
        action_add_quest->setText(QCoreApplication::translate("SolarusGui::MainWindow", "Add quest...", nullptr));
        action_exit->setText(QCoreApplication::translate("SolarusGui::MainWindow", "Exit", nullptr));
        action_play_quest->setText(QCoreApplication::translate("SolarusGui::MainWindow", "Play", nullptr));
#if QT_CONFIG(shortcut)
        action_play_quest->setShortcut(QCoreApplication::translate("SolarusGui::MainWindow", "F5", nullptr));
#endif // QT_CONFIG(shortcut)
        action_remove_quest->setText(QCoreApplication::translate("SolarusGui::MainWindow", "Remove quest", nullptr));
        action_stop_quest->setText(QCoreApplication::translate("SolarusGui::MainWindow", "Stop", nullptr));
        action_sound_volume->setText(QCoreApplication::translate("SolarusGui::MainWindow", "Sound volume", nullptr));
        action_music_volume->setText(QCoreApplication::translate("SolarusGui::MainWindow", "Music volume", nullptr));
        action_fullscreen->setText(QCoreApplication::translate("SolarusGui::MainWindow", "Fullscreen", nullptr));
#if QT_CONFIG(shortcut)
        action_fullscreen->setShortcut(QCoreApplication::translate("SolarusGui::MainWindow", "F11", nullptr));
#endif // QT_CONFIG(shortcut)
        action_zoom_x1->setText(QCoreApplication::translate("SolarusGui::MainWindow", "x1", nullptr));
        action_zoom_x2->setText(QCoreApplication::translate("SolarusGui::MainWindow", "x2", nullptr));
        action_zoom_x3->setText(QCoreApplication::translate("SolarusGui::MainWindow", "x3", nullptr));
        action_zoom_x4->setText(QCoreApplication::translate("SolarusGui::MainWindow", "x4", nullptr));
        action_mute->setText(QCoreApplication::translate("SolarusGui::MainWindow", "Mute", nullptr));
        action_about->setText(QCoreApplication::translate("SolarusGui::MainWindow", "About", nullptr));
#if QT_CONFIG(tooltip)
        add_button->setToolTip(QCoreApplication::translate("SolarusGui::MainWindow", "Add quest", nullptr));
#endif // QT_CONFIG(tooltip)
        add_button->setText(QCoreApplication::translate("SolarusGui::MainWindow", "Add quest...", nullptr));
#if QT_CONFIG(tooltip)
        remove_button->setToolTip(QCoreApplication::translate("SolarusGui::MainWindow", "Remove quest", nullptr));
#endif // QT_CONFIG(tooltip)
        remove_button->setText(QCoreApplication::translate("SolarusGui::MainWindow", "Remove quest", nullptr));
        quest_box_label->setText(QString());
        play_button->setText(QCoreApplication::translate("SolarusGui::MainWindow", "Play", nullptr));
        quest_title_value->setText(QCoreApplication::translate("SolarusGui::MainWindow", "Quest title", nullptr));
        quest_long_description_value->setText(QCoreApplication::translate("SolarusGui::MainWindow", "Description", nullptr));
        quest_author_label->setText(QCoreApplication::translate("SolarusGui::MainWindow", "Author:", nullptr));
        quest_author_value->setText(QCoreApplication::translate("SolarusGui::MainWindow", "Author", nullptr));
        quest_version_label->setText(QCoreApplication::translate("SolarusGui::MainWindow", "Version:", nullptr));
        quest_version_value->setText(QCoreApplication::translate("SolarusGui::MainWindow", "Version", nullptr));
        groupBox->setTitle(QCoreApplication::translate("SolarusGui::MainWindow", "Console", nullptr));
        menu_file->setTitle(QCoreApplication::translate("SolarusGui::MainWindow", "File", nullptr));
        menu_quest->setTitle(QCoreApplication::translate("SolarusGui::MainWindow", "Quest", nullptr));
        menu_video->setTitle(QCoreApplication::translate("SolarusGui::MainWindow", "Video", nullptr));
        menu_zoom->setTitle(QCoreApplication::translate("SolarusGui::MainWindow", "Zoom", nullptr));
        menu_audio->setTitle(QCoreApplication::translate("SolarusGui::MainWindow", "Audio", nullptr));
        menu_help->setTitle(QCoreApplication::translate("SolarusGui::MainWindow", "Help", nullptr));
    } // retranslateUi

};

} // namespace SolarusGui

namespace SolarusGui {
namespace Ui {
    class MainWindow: public Ui_MainWindow {};
} // namespace Ui
} // namespace SolarusGui

#endif // UI_MAIN_WINDOW_H
